/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 *  Authors: Iain Holmes <iain@prettypeople.org>
 *
 *  Copyright 2002 Iain Holmes
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef __MARLIN_SAMPLE_H__
#define __MARLIN_SAMPLE_H__

#include <glib-object.h>
#include <gst/gst.h>

#include <marlin/marlin-object.h>
#include <marlin/marlin-types.h>
#include <marlin/marlin-operation.h>
#include <marlin/marlin-undo-manager.h>
#include <marlin/marlin-utils.h>

#ifdef __cplusplus
extern "C" {
#pragma }
#endif

#define MARLIN_SAMPLE_TYPE (marlin_sample_get_type ())
#define MARLIN_SAMPLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MARLIN_SAMPLE_TYPE, MarlinSample))
#define MARLIN_SAMPLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CASE ((klass), MARLIN_SAMPLE_TYPE, MarlinSampleClass))
#define IS_MARLIN_SAMPLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MARLIN_SAMPLE_TYPE))
#define IS_MARLIN_SAMPLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), MARLIN_SAMPLE_TYPE))

typedef struct _MarlinSampleClass MarlinSampleClass;
typedef struct _MarlinSamplePrivate MarlinSamplePrivate;

typedef struct _MarlinSampleFade {
	float in_level; /* The level [0, 1] before the fade starts */
	float out_level; /* The level [0, 1] after the fade ends */
	guint64 fade_start; /* The sample position the fade starts at */
	guint64 fade_end; /* The position the fade ends at */
} MarlinSampleFade;

struct _MarlinSample {
	MarlinObject parent;

	MarlinSamplePrivate *priv;
};

struct _MarlinSampleClass {
	MarlinObjectClass parent_class;

	void (*data_changed) (MarlinSample *sample,
			      MarlinRange *range);
};

GType marlin_sample_get_type (void);
MarlinSample *marlin_sample_new (void);
MarlinSample *marlin_sample_new_from_sample_with_range (MarlinSample *src,
							MarlinRange *range,
							GError **error);
MarlinSample *marlin_sample_new_from_sample (MarlinSample *src,
					     GError **error);

MarlinChannel *marlin_sample_get_channel (MarlinSample *sample,
					  guint channel_num);

/* Sample editting functions */
gboolean marlin_sample_delete_range (MarlinSample *sample,
				     MarlinOperation *operation,
				     MarlinRange *range,
				     MarlinUndoContext *ctxt,
				     GError **error);
gboolean marlin_sample_clear_range (MarlinSample *sample,
				    MarlinOperation *operation,
				    MarlinRange *range,
				    MarlinUndoContext *ctxt,
				    GError **error);
gboolean marlin_sample_crop_range (MarlinSample *sample,
				   MarlinOperation *operation,
				   MarlinRange *range,
				   MarlinUndoContext *ctxt,
				   GError **error);

gboolean marlin_sample_swap_channels (MarlinSample *sample,
				      MarlinOperation *operation,
				      MarlinUndoContext *ctxt,
				      GError **error);
gboolean marlin_sample_mix (MarlinSample *dest,
			    MarlinSample *src,
			    double src_db,
			    double dest_db,
			    MarlinRange *range,
			    gboolean clip,
			    MarlinOperation *operation,
			    MarlinUndoContext *ctxt,
			    GError **error);
gboolean marlin_sample_insert (MarlinSample *dest,
			       MarlinSample *src,
			       guint64 position,
			       MarlinCoverage coverage,
			       MarlinUndoContext *ctxt,
			       GError **error);
gboolean marlin_sample_invert_range (MarlinSample *sample,
				     MarlinRange *range,
				     MarlinUndoContext *ctxt,
				     GError **error);
gboolean marlin_sample_adjust_volume_range (MarlinSample *sample,
					    float db,
					    MarlinRange *range,
					    MarlinUndoContext *ctxt,
					    GError **error);
gboolean marlin_sample_insert_silence (MarlinSample *sample,
				       MarlinOperation *operation,
				       guint64 position,
				       guint64 length,
				       MarlinUndoContext *ctxt,
				       GError **error);

gboolean marlin_sample_remove_channel (MarlinSample *sample,
				       MarlinChannelPosition channel,
				       gboolean mix,
				       MarlinOperation *operation,
				       MarlinUndoContext *ctxt,
				       GError **error);
gboolean marlin_sample_add_channel (MarlinSample *sample,
				    MarlinChannelPosition channel,
				    gboolean clone,
				    MarlinOperation *operation,
				    MarlinUndoContext *ctxt,
				    GError **error);
gboolean marlin_sample_fade (MarlinSample *sample,
			     MarlinSampleFade *fade,
			     MarlinOperation *operation,
			     MarlinUndoContext *ctxt,
			     GError **error);

guint64 marlin_sample_next_zero (MarlinSample *sample,
				 guint64 position,
				 MarlinCoverage coverage);
guint64 marlin_sample_previous_zero (MarlinSample *sample,
				     guint64 position,
				     MarlinCoverage coverage);

void marlin_sample_read_lock (MarlinSample *sample);
void marlin_sample_read_unlock (MarlinSample *sample);

#ifdef __cplusplus
}
#endif

#endif
